from enum import Enum


class OPERATIONS(Enum):
    ADDED = "ADDED"
    MODIFIED = "MODIFIED"
    REMOVED = "REMOVED"

    @classmethod
    def choices(cls):
        return tuple((i.name, i.value) for i in cls)



from calculator.helpers import if_none_raise, if_empty_string_raise
from enum import Enum


class ConversionOrder(Enum):
    Same = 1
    FromIsBase = 2
    ToIsBase = 3


class CurrencyConverter:

    def __init__(self, exchange_rate_item):
        self.exchange_rate_item = exchange_rate_item

    def convert_value(self, value, from_currency, to_currency):
        conversion_order = self.conversion_is_possible(from_currency, to_currency)

        if conversion_order == conversion_order.Same:
            return value
        elif conversion_order == conversion_order.FromIsBase:
            ratio = getattr(self.exchange_rate_item.rates,to_currency)
            return value * ratio
        elif conversion_order == conversion_order.ToIsBase:
            ratio = getattr(self.exchange_rate_item.rates, from_currency)
            return value / ratio

    def conversion_is_possible(self, from_currency, to_currency):
        if_none_raise(self.exchange_rate_item)
        if_empty_string_raise(from_currency)
        if_empty_string_raise(to_currency)

        if from_currency == to_currency:
            return ConversionOrder.Same
        elif from_currency == self.exchange_rate_item.base:
            return ConversionOrder.FromIsBase
        elif to_currency == self.exchange_rate_item.base:
            return ConversionOrder.ToIsBase



import logging


def if_none_raise_with_id(_id, trans):
    if trans is None:
        raise ValueError(str(_id) + " does not exist ")


def if_none_raise(trans):
    if trans is None:
        raise ValueError( " does not exist ")


def if_empty_string_raise(value):
    if value is None:
        raise ValueError( " does not exist ")
    if value == '':
        raise ValueError(" does not exist ")


def log_error(exception, pk_id, web_method_name, cfg):
    logging.basicConfig(filename=cfg.LOGS_PATH, level=logging.ERROR)
    logging.error(str(pk_id) + ' ' + str(web_method_name) + ' ' + str(exception))





from enum import Enum


class TRANSACTION_TYPES(Enum):
    Deposit = "Deposit"
    Trade = "Trade"
    Withdrawal = "Withdrawal"

    @classmethod
    def choices(cls):
        return tuple((i.name, i.value) for i in cls)



from mongoengine import *


class Rates(EmbeddedDocument):
    meta = {'strict': False}
    AED = FloatField()
    AFN = FloatField()
    ALL = FloatField()
    AMD = FloatField()
    ANG = FloatField()
    AOA = FloatField()
    ARS = FloatField()
    AUD = FloatField()
    AWG = FloatField()
    AZN = FloatField()
    BAM = FloatField()
    BBD = FloatField()
    BDT = FloatField()
    BGN = FloatField()
    BHD = FloatField()
    BIF = FloatField()
    BMD = FloatField()
    BND = FloatField()
    BOB = FloatField()
    BRL = FloatField()
    BSD = FloatField()
    BTC = FloatField()
    BTN = FloatField()
    BWP = FloatField()
    BYN = FloatField()
    BYR = FloatField()
    BZD = FloatField()
    CAD = FloatField()
    CDF = FloatField()
    CHF = FloatField()
    CLF = FloatField()
    CLP = FloatField()
    CNY = FloatField()
    COP = FloatField()
    CRC = FloatField()
    CUC = FloatField()
    CUP = FloatField()
    CVE = FloatField()
    CZK = FloatField()
    DJF = FloatField()
    DKK = FloatField()
    DOP = FloatField()
    DZD = FloatField()
    EGP = FloatField()
    ERN = FloatField()
    ETB = FloatField()
    EUR = FloatField()
    FJD = FloatField()
    FKP = FloatField()
    GBP = FloatField()
    GEL = FloatField()
    GGP = FloatField()
    GHS = FloatField()
    GIP = FloatField()
    GMD = FloatField()
    GNF = FloatField()
    GTQ = FloatField()
    GYD = FloatField()
    HKD = FloatField()
    HNL = FloatField()
    HRK = FloatField()
    HTG = FloatField()
    HUF = FloatField()
    IDR = FloatField()
    ILS = FloatField()
    IMP = FloatField()
    INR = FloatField()
    IQD = FloatField()
    IRR = FloatField()
    ISK = FloatField()
    JEP = FloatField()
    JMD = FloatField()
    JOD = FloatField()
    JPY = FloatField()
    KES = FloatField()
    KGS = FloatField()
    KHR = FloatField()
    KMF = FloatField()
    KPW = FloatField()
    KRW = FloatField()
    KWD = FloatField()
    KYD = FloatField()
    KZT = FloatField()
    LAK = FloatField()
    LBP = FloatField()
    LKR = FloatField()
    LRD = FloatField()
    LSL = FloatField()
    LTL = FloatField()
    LVL = FloatField()
    LYD = FloatField()
    MAD = FloatField()
    MDL = FloatField()
    MGA = FloatField()
    MKD = FloatField()
    MMK = FloatField()
    MNT = FloatField()
    MOP = FloatField()
    MRO = FloatField()
    MUR = FloatField()
    MVR = FloatField()
    MWK = FloatField()
    MXN = FloatField()
    MYR = FloatField()
    MZN = FloatField()
    NAD = FloatField()
    NGN = FloatField()
    NIO = FloatField()
    NOK = FloatField()
    NPR = FloatField()
    NZD = FloatField()
    OMR = FloatField()
    PAB = FloatField()
    PEN = FloatField()
    PGK = FloatField()
    PHP = FloatField()
    PKR = FloatField()
    PLN = FloatField()
    PYG = FloatField()
    QAR = FloatField()
    RON = FloatField()
    RSD = FloatField()
    RUB = FloatField()
    RWF = FloatField()
    SAR = FloatField()
    SBD = FloatField()
    SCR = FloatField()
    SDG = FloatField()
    SEK = FloatField()
    SGD = FloatField()
    SHP = FloatField()
    SLL = FloatField()
    SOS = FloatField()
    SRD = FloatField()
    STD = FloatField()
    SVC = FloatField()
    SYP = FloatField()
    SZL = FloatField()
    THB = FloatField()
    TJS = FloatField()
    TMT = FloatField()
    TND = FloatField()
    TOP = FloatField()
    TRY = FloatField()
    TTD = FloatField()
    TWD = FloatField()
    TZS = FloatField()
    UAH = FloatField()
    UGX = FloatField()
    USD = FloatField()
    UYU = FloatField()
    UZS = FloatField()
    VEF = FloatField()
    VND = FloatField()
    VUV = FloatField()
    WST = FloatField()
    XAF = FloatField()
    XAG = FloatField()
    XAU = FloatField()
    XCD = FloatField()
    XDR = FloatField()
    XOF = FloatField()
    XPF = FloatField()
    YER = FloatField()
    ZAR = FloatField()
    ZMK = FloatField()
    ZMW = FloatField()
    ZWL = FloatField()


class exchange_rates(Document):
    meta = {'strict': False}
    _id = IntField(db_field='_id')
    success = BooleanField()
    base = StringField()
    rates = EmbeddedDocumentField(Rates, db_field='rates')
    date = StringField()


from cryptomodel.cryptostore_symbol_value import total_symbol_value


class BalanceCalculator:

    def __init__(self, transactions, symbol_rates, exchange_rates, preferred_currency, upper_bound_symbol_rates_date,
                 upper_bound_transaction_date):
        self.transactions = transactions
        self.symbol_rates = symbol_rates
        self.exchange_rates = exchange_rates
        self.preferred_currency = preferred_currency
        self.upper_bound_symbol_rates_date = upper_bound_symbol_rates_date
        self.upper_bound_transaction_date = upper_bound_transaction_date

    def compute(self, user_id, date):
        tsv = total_symbol_value(user_id=user_id, date_time_calculated=date,
                                 preferred_currency=self.preferred_currency,
                                 symbol_prices=self.symbol_rates,
                                 exchange_rates=self.exchange_rates,
                                 upper_bound_symbol_rates_date=self.upper_bound_symbol_rates_date,
                                 upper_bound_transaction_date=self.upper_bound_transaction_date)
        return tsv.create(self.transactions)
